@@ -199,7 +199,7 @@ void tilda_window_toggle_transparency (tilda_window *tw)
 #endif
 }
 
-gint toggle_searchbar_cb (tilda_window *tw)
+gint tilda_window_toggle_searchbar (tilda_window *tw)
 {
     DEBUG_FUNCTION ("toggle_searbar");
     DEBUG_ASSERT (tw != NULL);
@@ -319,15 +319,18 @@ gint tilda_window_next_tab (tilda_window *tw)
 
     int num_pages;
     int current_page;
+    GtkNotebook *notebook;
+
+    notebook = GTK_NOTEBOOK (tw->notebook);
 
     /* If we are on the last page, go to first page */
-    num_pages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (tw->notebook));
-    current_page = gtk_notebook_get_current_page (GTK_NOTEBOOK (tw->notebook));
+    num_pages = gtk_notebook_get_n_pages (notebook);
+    current_page = gtk_notebook_get_current_page (notebook);
 
     if ((num_pages - 1) == current_page)
-      gtk_notebook_set_current_page (GTK_NOTEBOOK (tw->notebook), 0);
+      gtk_notebook_set_current_page (notebook, 0);
     else
-      gtk_notebook_next_page (GTK_NOTEBOOK (tw->notebook));
+      gtk_notebook_next_page (notebook);
 
     // It worked. Having this return GDK_EVENT_STOP makes the callback not carry the
     // keystroke into the vte terminal widget.
@@ -341,14 +344,17 @@ gint tilda_window_prev_tab (tilda_window *tw)
 
     int num_pages;
     int current_page;
+    GtkNotebook *notebook;
 
-    num_pages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (tw->notebook));
-    current_page = gtk_notebook_get_current_page (GTK_NOTEBOOK (tw->notebook));
+    notebook = GTK_NOTEBOOK (tw->notebook);
+
+    num_pages = gtk_notebook_get_n_pages (notebook);
+    current_page = gtk_notebook_get_current_page (notebook);
 
     if (current_page == 0)
-        gtk_notebook_set_current_page (GTK_NOTEBOOK (tw->notebook), (num_pages -1));
+      gtk_notebook_set_current_page (notebook, (num_pages - 1));
     else
-      gtk_notebook_prev_page (GTK_NOTEBOOK (tw->notebook));
+      gtk_notebook_prev_page (notebook);
 
     // It worked. Having this return GDK_EVENT_STOP makes the callback not carry the
     // keystroke into the vte terminal widget.
@@ -366,19 +372,21 @@ static gint move_tab (tilda_window *tw, int direction)
     int current_page_index;
     int new_page_index;
     GtkWidget* current_page;
+    GtkNotebook* notebook;
 
-    num_pages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (tw->notebook));
+    notebook = GTK_NOTEBOOK (tw->notebook);
+
+    num_pages = gtk_notebook_get_n_pages (notebook);
 
     if (num_pages > 1) {
-        current_page_index = gtk_notebook_get_current_page (GTK_NOTEBOOK (tw->notebook));
-        current_page = gtk_notebook_get_nth_page (GTK_NOTEBOOK (tw->notebook),
+        current_page_index = gtk_notebook_get_current_page (notebook);
+        current_page = gtk_notebook_get_nth_page (notebook,
                                                   current_page_index);
 
         /* wrap over if new_page_index over-/underflows */
         new_page_index = (current_page_index + direction) % num_pages;
 
-        gtk_notebook_reorder_child (GTK_NOTEBOOK (tw->notebook), current_page,
-                                    new_page_index);
+        gtk_notebook_reorder_child (notebook, current_page, new_page_index);
     }
 
     // It worked. Having this return GDK_EVENT_STOP makes the callback not carry the
@@ -665,7 +673,7 @@ static gint tilda_window_setup_keyboard_accelerators (tilda_window *tw)
     tilda_add_config_accelerator_by_path("fullscreen_key", "<tilda>/context/Toggle Fullscreen", G_CALLBACK(toggle_fullscreen_cb),           tw);
     tilda_add_config_accelerator_by_path("quit_key",       "<tilda>/context/Quit",              G_CALLBACK(gtk_main_quit),                  tw);
     tilda_add_config_accelerator_by_path("toggle_transparency_key", "<tilda>/context/Toggle Transparency", G_CALLBACK(toggle_transparency_cb),      tw);
-    tilda_add_config_accelerator_by_path("toggle_searchbar_key", "<tilda>/context/Toggle Searchbar", G_CALLBACK(toggle_searchbar_cb),      tw);
+    tilda_add_config_accelerator_by_path("toggle_searchbar_key", "<tilda>/context/Toggle Searchbar", G_CALLBACK(tilda_window_toggle_searchbar),     tw);
 
     tilda_add_config_accelerator_by_path("nexttab_key",      "<tilda>/context/Next Tab",        G_CALLBACK(tilda_window_next_tab),          tw);
     tilda_add_config_accelerator_by_path("prevtab_key",      "<tilda>/context/Previous Tab",    G_CALLBACK(tilda_window_prev_tab),          tw);
@@ -756,12 +764,12 @@ static void tilda_window_search (G_GNUC_UNUSED GtkWidget *widget, tilda_window *
     g_regex_unref(regex);
 }
 
-static void tilda_window_search_forward_cb (GtkButton *button, tilda_window *tw) {
+static void tilda_window_search_forward_cb (GtkWidget *button, tilda_window *tw) {
     /* The default is to search forward */
     tilda_window_search (button, tw, FALSE);
 }
 
-static void tilda_window_search_backward_cb (GtkButton *button, tilda_window *tw) {
+static void tilda_window_search_backward_cb (GtkWidget *button, tilda_window *tw) {
     tilda_window_search (button, tw, TRUE);
 }
 
@@ -826,6 +834,8 @@ static tilda_search *tilda_search_box_init(tilda_window *tw)
     tilda_search *search = malloc(sizeof(tilda_search));
 
     search->search_box = GTK_WIDGET(gtk_builder_get_object (gtk_builder, "search_box"));
+    gtk_widget_set_name (GTK_WIDGET (search->search_box), "search");
+
     DEBUG_ASSERT(search->search_box != NULL);
     search->button_next = GTK_WIDGET (gtk_builder_get_object (gtk_builder, "button_search_next"));
     search->button_prev = GTK_WIDGET (gtk_builder_get_object (gtk_builder, "button_search_prev"));
@@ -849,17 +859,18 @@ static void page_reordered_cb (GtkNotebook  *notebook,
                         guint         page_num,
                         tilda_window *tw) {
     DEBUG_FUNCTION ("page_reordered_cb");
-    GList *terms = tw->terms;
+    GList *terminals;
+    tilda_term *tilda_term;
+    guint i;
 
-    /* We use the VteTerminal pointer relative to the current tab's child widget to find the correct
-     * tilda_term structure in the list of terminals. The assumption here is, that the VteTerminal
-     * is either the first or second widget in the container (depending on the location of the scrollbar) */
-    for (GList *item = terms; item != NULL; item = item->next) {
-        tilda_term *current_term = item->data;
-        if (current_term->hbox == child) {
-            terms = g_list_remove (terms, current_term);
-            terms = g_list_insert (terms, current_term, page_num);
-            tw->terms = terms;
+    terminals = tw->terms;
+
+    for (i = 0; i < g_list_length (terminals); i++)
+    {
+        tilda_term = g_list_nth_data (terminals, i);
+        if (tilda_term->hbox == child) {
+            terminals = g_list_remove (terminals, tilda_term);
+            tw->terms = g_list_insert (terminals, tilda_term, page_num);
             break;
         }
     }
@@ -938,6 +949,8 @@ gboolean tilda_window_init (const gchar *config_file, const gint instance, tilda
     tilda_window_set_icon (tw, g_build_filename (DATADIR, "pixmaps", "tilda.png", NULL));
     tilda_window_setup_alpha_mode (tw);
 
+    gtk_widget_set_app_paintable (GTK_WIDGET (tw->window), TRUE);
+
     /* Add keyboard accelerators */
     tw->accel_group = NULL; /* We can redefine the accelerator group from the wizard; this shows that it's our first time defining it. */
     tilda_window_setup_keyboard_accelerators (tw);
@@ -1180,5 +1193,5 @@ gint tilda_window_close_tab (tilda_window *tw, gint tab_index, gboolean force_ex
     /* Free the terminal, we are done with it */
     tilda_term_free (tt);
 
-    return 0;
+    return TRUE;
 }
